/*
 *  Jomar3D, a lobster-running-to-catch game.
 *  Copyright (C) 2006-2008 Jonathan Michalon <studios.chalmion@no-log.org>.
 * 
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <math.h>
#include <stdio.h>
#include <GL/glut.h>
#include "jomar3D.h"
#include "draw.h"

/* set the viewport */

void
setview (void)
{
  glViewport (0, 0, winwidth, winheight);
  gluPerspective ( /* field of view in degree */ 80.0,   /* aspect ratio */
                  ratio,
                                    /* Z near */ 0.5,
                                    /* Z far */
                  sqrt ((T_X * T_C) * (T_X * T_C) +
                        (T_Y * T_C) * (T_Y * T_C)));
}

/* take place in the given window */

void
reshape (int width, int height)
{
#if (DEBUG)
  printf ("width, height %d %d\n", width, height);
#endif
  winwidth = width;
  winheight = height;
  ratio = (float) (width) / (float) (height);
  glMatrixMode (GL_PROJECTION);
  glLoadIdentity ();
  setview ();
  glMatrixMode (GL_MODELVIEW);
  if (!g_winmode && !winuserdef)
    glutPositionWindow ((glutGet (GLUT_SCREEN_WIDTH) - width) / 2.,
                        (glutGet (GLUT_SCREEN_HEIGHT) - height) / 2.);
  createsmap ();
  glutPostRedisplay ();
}

/* set all what must be to launch the game -- graphic part */

/* manage the window: full screen of window */

void
windowmanage (void)
{
  char gmsize[20];

/* if it is possible to resize to those crazy values, we cannot resize... then don't */
  sprintf (gmsize, "123x456");
  glutGameModeString (gmsize);
  if (glutGameModeGet (GLUT_GAME_MODE_POSSIBLE))
    {
      sprintf (gmsize, "%dx%d", glutGet (GLUT_SCREEN_WIDTH),
               glutGet (GLUT_SCREEN_HEIGHT));
    }
  else
    {
      sprintf (gmsize, "%dx%d", winwidth, winheight);
    }
  glutGameModeString (gmsize);
//glutGameModeString( "1600x1200" );
//printf ("Possible Game Mode? %d\n", glutGameModeGet(GLUT_GAME_MODE_POSSIBLE));
//if (!glutGameModeGet(GLUT_GAME_MODE_POSSIBLE)) {
//    printf("Resolution of this window not possible for fullscreen\n");
//    g_winmode = !g_winmode ;
//    return;
//}
#if (DEBUG)
  printf ("game mode string %s\n", gmsize);
#endif

  if (g_winmode)
    {
#if (DEBUG)
      printf ("Entering game mode\n");
#endif
      glutDestroyWindow (glutGetWindow ());
      glutEnterGameMode ();
    }
  else
    {
#if (DEBUG)
      printf ("Leaving game mode\n");
#endif
      if (ratio != 0.)
        {
          glutLeaveGameMode ();
          glutCreateWindow ("jomar3D");
        }
      else
        {
          glutCreateWindow ("jomar3D");
        }
    }

  glutSetIconTitle ("jomar3D");
  glutSetWindowTitle ("jomar3D");

/* reset the fps average because it has no reason to stay from the last configuration */
  average = 0;

/* the textures are lost when new gl context so reload them all */
#if (MYFREEGLUT == 0)
  loadtextures (1);             // not if we hold the last context (freeglut_ext), option
#endif

}

/* timer called function to hide the cursor */
void
nocur (int dummy)
{
  (void) dummy;
  glutSetCursor (GLUT_CURSOR_NONE);
}
