/*
 *  Jomar3D, a lobster-running-to-catch game.
 *  Copyright (C) 2006-2008 Jonathan Michalon <studios.chalmion@no-log.org>.
 * 
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


#include <stdlib.h>
#include <string.h>

#include "utils.h"

/* return the extension of a filename or NULL -- (C) Colomban Wendling GPL v.3 */
const char *
getext (const char *str)
{
  const char *ext = NULL;

  if ((ext = strrchr (str, '.')) != NULL)
    {
      if (ext[1] == '\0')
        return NULL;
      else
        return ++ext;
    }

  return NULL;
}

/* malloc and fill a pointer with a string */
char *
puts2p (const char *string)
{
  char *ret;

  ret = malloc (strlen (string) * sizeof (string));
  if (ret != NULL)
    strcpy (ret, string);
  return ret;
}

/* scandir doesn't exist in the mingw32 (windows) so make it (simplified) */
int
myscandir (const char *dir, struct dirent ***namelist,
           int (*select) (const struct dirent *))
{
  DIR *d;
  struct dirent *entry;
  register int i = 0;
  size_t entrysize;

  if ((d = opendir (dir)) == NULL)
    return (-1);

  *namelist = NULL;
  while ((entry = readdir (d)) != NULL)
    {
      if (select == NULL || (select != NULL && (*select) (entry)))
        {
          *namelist = (struct dirent **) realloc ((void *) (*namelist),
                                                  (size_t) ((i +
                                                             1) *
                                                            sizeof (struct
                                                                    dirent
                                                                    *)));
          if (*namelist == NULL)
            return (-1);
          entrysize =
            sizeof (struct dirent) - sizeof (entry->d_name) +
            strlen (entry->d_name) + 1;
          (*namelist)[i] = (struct dirent *) malloc (entrysize);
          if ((*namelist)[i] == NULL)
            return (-1);
          memcpy ((*namelist)[i], entry, entrysize);
          i++;
        }
    }
  if (closedir (d))
    return (-1);
  if (i == 0)
    return (-1);

  return (i);
}
